{ Program:   VbxInfo
  Version:   1.00
  Purpose:   program to extract information from VBX files
  Uses:      BIVBX10.DLL from the BC4 package

  Developer: Peter Sawatzki (ps)
             Buchenhof 3, D58091 Hagen, Germany
 CompuServe: 100031,3002

  Date:     Author:
  02/26/94  ps       written

  Copyright (c) 1994 Peter Sawatzki. All Rights Reserved.
}
Program VbxInfo;
Uses
  WinTypes,
  WinProcs,
  Strings,
  Objects,
  oWindows,
  oDialogs,
  oStdWnds,
  oMemory,
  CommDlg,
{$IfDef Debug} Debug, {$EndIf}
  Vbx;
Const
  VBXvalidation: tVbxValidation = cVbxValidation;

Type
  pInfoWindow = ^tInfoWindow;
  tInfoWindow = Object(tFileWindow)
    Constructor Init(aParent: pWindowsObject; aTitle: pChar);
    Procedure SetupWindow; Virtual;
    Function GetFileName (aFn: pChar): pChar;
    Function GetPascalFileName (aFn: pChar): pChar;
    Function CanClear: Boolean; Virtual;
    Procedure cmFileOpen (Var Msg: tMessage); Virtual cm_First+cm_FileOpen;
    Procedure cmFileSaveAs (Var Msg: tMessage); Virtual cm_First+cm_FileSaveAs;
    Function SavePascal: Boolean;
    Procedure RetrieveInfo (aVBXName: pChar);
  End;

Constructor tInfoWindow.Init (aParent: pWindowsObject; aTitle: pChar);
Begin
  Inherited Init(aParent, aTitle, Nil);
  Attr.Menu:= LoadMenu(hInstance, 'FileCommands');
  DeleteMenu(Attr.Menu, cm_FileNew, mf_ByCommand);
  DeleteMenu(Attr.Menu, cm_FileSave, mf_ByCommand);
  ModifyMenu(Attr.Menu, cm_FileOpen, mf_ByCommand, cm_FileOpen, 'Load VBX file...');
  ModifyMenu(Attr.Menu, cm_FileSaveAs, mf_ByCommand, cm_FileSaveAs, 'Save Pascal Source as...');
End;

Procedure tInfoWindow.SetupWindow;
Begin
  Inherited SetupWindow;
  SendMessage(Editor^.hWindow, wm_SetFont, GetStockObject(Oem_Fixed_Font), 0)
End;

Function tInfoWindow.GetFileName (aFn: pChar): pChar;
Var
  OpenFN      : tOpenFileName;
  Filter      : array[0..100] of Char;
  StartDir,
  FName,
  FullFileName: array[0..100] Of Char;
Begin
  GetFileName:= aFn;
  StrCopy(FullFileName, '');

  GetWindowsDirectory(StartDir, SizeOf(StartDir));
  StrCat(StartDir, '\system');

  FillChar(Filter, SizeOf(Filter), #0);  { Set up for double null at end }
  StrCopy(Filter, 'VBX files (*.VBX)');
  StrCopy(@Filter[StrLen(Filter)+1], '*.VBX');

  FillChar(OpenFN, SizeOf(TOpenFileName), #0);
  With OpenFN Do Begin
    hInstance     := System.hInstance;
    hwndOwner     := hWindow;
    lpstrDefExt   := 'VBX';
    lpstrTitle    := 'Load VBX file';

    lpstrFile     := FullFileName;
    lpstrFilter   := Filter;
    lpstrFileTitle:= FName;
    lpstrInitialDir:= StartDir;
    flags         := ofn_FileMustExist Or ofn_HideReadOnly;
    lStructSize   := SizeOf(tOpenFileName);
    nFilterIndex  := 1;       {Index into Filter String in lpstrFilter}
    nMaxFile      := SizeOf(FullFileName);
  End;
  If GetOpenFileName(OpenFN) Then
    StrCopy(aFn, FullFileName)
  Else
    StrCopy(aFn, '')
End;

Function tInfoWindow.GetPascalFileName (aFn: pChar): pChar;
Var
  OpenFN      : tOpenFileName;
  Filter      : array[0..100] of Char;
  FName,
  FullFileName: array[0..100] Of Char;
Begin
  GetPascalFileName:= aFn;
  StrCopy(FullFileName, aFn);

  FillChar(Filter, SizeOf(Filter), #0);  { Set up for double null at end }
  StrCopy(Filter, 'Pascal units (*.PAS)');
  StrCopy(@Filter[StrLen(Filter)+1], '*.PAS');

  FillChar(OpenFN, SizeOf(TOpenFileName), #0);
  With OpenFN Do Begin
    hInstance     := System.hInstance;
    hwndOwner     := hWindow;
    lpstrDefExt   := 'PAS';
    lpstrTitle    := 'Save as Pascal unit';

    lpstrFile     := FullFileName;
    lpstrFilter   := Filter;
    lpstrFileTitle:= FName;
    flags         := ofn_HideReadOnly;
    lStructSize   := SizeOf(tOpenFileName);
    nFilterIndex  := 1;       {Index into Filter String in lpstrFilter}
    nMaxFile      := SizeOf(FullFileName);
  End;
  If GetSaveFileName(OpenFN) Then
    StrCopy(aFn, FullFileName)
  Else
    StrCopy(aFn, '')
End;

Function tInfoWindow.CanClear: Boolean;
Var
  S: Array[0..130] of Char;
  P: pChar;
begin
  CanClear := True;
  If Editor^.IsModified Then Begin
    If Assigned(FileName) Then Begin
      p:= FileName;
      WVSPrintF(S, 'File "%s" not saved.  Save?', P);
    End Else
      StrCopy(S, 'Untitled file not saved. Save?');
    Case MessageBox(hWindow, S, 'File not saved', mb_YesNoCancel or mb_IconQuestion) Of
      id_Yes: CanClear:= SavePascal;
      id_Cancel: CanClear:= False
    Else
      CanClear:= True
    End
  End
End;

Procedure tInfoWindow.cmFileOpen (Var Msg: tMessage);
Var
  TmpName: Array[0..67] Of Char;
Begin
  If CanClear And (StrLen(GetFileName(Tmpname))>0) Then
    RetrieveInfo(TmpName)
End;

Procedure tInfoWindow.cmFileSaveAs (Var Msg: tMessage);
Begin
  SavePascal
End;

Function tInfoWindow.SavePascal: Boolean;
Var
  TmpName: array[0..67] of Char;
Begin
  SavePascal:= False;
  If Assigned(FileName) Then
    StrCopy(TmpName, FileName)
  Else
    TmpName[0]:= #0;
  If StrLen(GetPascalFileName(TmpName))>0 Then Begin
    SetFileName(TmpName);
    Write;
    SavePascal:= True
  End
End;

Type
  pVbxClassInfo = ^tVbxClassInfo;
  tVbxClassInfo = Record
    usVersion: Word;                { VB version used by control                    }
    fl: LongInt;                    { Bitfield structure                            }
    pctlproc: pointer;              { the control proc.                             }
    fsClassStyle: Word;             { window class style                            }
    flWndStyle: LongInt;            { default window style                          }
    cbCtlExtra: Word;               { # bytes alloc'd for HCTL structure            }
    idBmpPalette: Word;             { BITMAP id for tool palette                    }
    npszDefCtlName: Word;           { offset of default control name prefix         }
    npszClassName: Word;            { offset of Visual Basic class name             }
    npszParentClassName: Word;      { offset of Parent window class if subclassed   }
    npproplist: Word;               { offset of Property list                       }
    npeventlist: Word;              { offset of Event list                          }
    nDefProp: Byte;                 { index of default property                     }
    nDefEvent: Byte;                { index of default event                        }
    nValueProp: byte;
    usControlVersion: word
  End;

  pVbxClass = ^tVbxClass;
  tVbxClass = Record
    dummy: Array[0..5] Of Byte;
    ClassInfo: pVbxClassInfo
  End;

  pDumpControl = ^tDumpControl;
  tDumpControl = Object(tVbxControl)
    BufPtr: pChar;
    VbxBaseName: Array[0..67] Of Char;
    Constructor Init (aParent: pInfoWindow; aVbxName, aVbxClass: pChar; aPtr: pChar);
    Function GetEventId (Dst: pChar; Index: Integer): pChar;
    Procedure DumpProperties;
    Procedure DumpEvents;
    Procedure DumpObjectDefinition;
    Procedure DumpObjectImplementation;
  End;

  NumStr = Array[0..30] Of Char;

  Function L2Str (Dst: pChar; aLong: LongInt): pChar;
  Begin
    L2Str:= Dst;
    Str(aLong, NumStr(Pointer(Dst)^))
  End;

  Function Str2Id (Dst, Src: pChar): pChar;
  Begin
    Str2Id:= Dst;
    While Src[0]<>#0 Do Begin
      Dst[0]:= Src[0];
      Case Src[0] Of
        'a'..'z',
        'A'..'Z',
        '0'..'9',
        '_':      Inc(Dst)
      End;
      Inc(Src)
    End;
    Dst[0]:= #0
  End;

Function StrJustName (Dst, Src: pChar): pChar;
Var
  p: pChar;
Begin
  p:= StrRScan(Src,'\');
  If Not Assigned(p) Then
    p:= StrRScan(Src,':');
  If Not Assigned(p) Then
    p:= Src
  Else
    Inc(p);
  StrJustName:= StrCopy(Dst, p)
End;

Constructor tDumpControl.Init (aParent: pInfoWindow; aVbxName, aVbxClass: pChar; aPtr: pChar);
Begin
  Inherited Init (aParent, 123, aVbxName, aVbxClass, 'TestCtl', 1, 1, 100, 100, 0, Nil);
  BufPtr:= aPtr;
  StrJustName(VbxBaseName, aVbxName)
End;

Function tDumpControl.GetEventId (Dst: pChar; Index: Integer): pChar;
Begin
  GetEventId:= Str2Id(Dst, GetEventName(Index))
End;

Procedure tDumpControl.DumpProperties;
Var
  aLine: Array[0..100] Of Char;
  i: Integer;
Begin
  BufPtr:= StrEnd(StrCat(BufPtr, 'Const'#13#10'{-Properties}'#13#10));
  For i:= 0 To GetNumProps-1 Do Begin
    StrCat(StrCat(StrCopy(aLine,'  Prop_'), VbxClass),'_');
    Str2Id(StrEnd(aLine), GetPropName(i));
    While StrLen(aLine)<40 Do StrCat(aLine, ' ');
    StrCat(aLine, ' = '); If i<10 Then StrCat(aLine, ' ');
    StrCat(L2Str(StrEnd(aLine), i),';'#13#10);
    BufPtr:= StrEnd(StrCat(BufPtr, aLine))
  End
End;

Procedure tDumpControl.DumpEvents;
Var
  aLine: Array[0..100] Of Char;
  i: Integer;
Begin
  BufPtr:= StrEnd(StrCat(BufPtr, #13#10'{-Events}'#13#10));
  For i:= 0 To GetNumEvents-1 Do Begin
    StrCat(StrCat(StrCopy(aLine,'  ev_'), VbxClass),'_');
    GetEventId(StrEnd(aLine),i);
    While StrLen(aLine)<40 Do StrCat(aLine, ' ');
    StrCat(aLine, ' = '); If i<10 Then StrCat(aLine, ' ');
    StrCat(L2Str(StrEnd(aLine), i),';'#13#10);
    BufPtr:= StrEnd(StrCat(BufPtr, aLine))
  End
End;

Procedure tDumpControl.DumpObjectDefinition;
Var
  i: Integer;
  Tmp: Array[0..67] Of Char;
Begin
  BufPtr:= StrEnd(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(BufPtr,
    'Type'#13#10+
    '{- '), VbxClass), ' }'#13#10+
    '  p'), VbxClass), ' = ^t'), VbxClass), ';'#13#10'  t'), VbxClass),
    ' = Object(tVbxControl)'#13#10+
    '    Constructor Init (aParent: pWindowsObject; anId: Integer; Title: pChar;'#13#10+
    '                      x,y,w,h: Integer; Len: LongInt; Data: Pointer);'#13#10+
    '    Constructor InitResource(aParent: pWindowsObject; anId: Integer);'#13#10));
  For i:= 0 To GetNumEvents-1 Do
    BufPtr:= StrEnd(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(BufPtr,
      '    Procedure ev'), GetEventId(Tmp, i)),
      ' (Var Event: tVbxEvent); Virtual ev_First+ev_'), VbxClass), '_'), GetEventId(Tmp, i)), ';'#13#10));
  BufPtr:= StrEnd(StrCat(BufPtr, '  End;'#13#10))
End;

Procedure tDumpControl.DumpObjectImplementation;
Var
  i: Integer;
  Tmp: Array[0..67] Of Char;
Begin
  BufPtr:= StrEnd(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(BufPtr,
    '{- '), VbxClass), ' }'#13#10+
    'Constructor t'), VbxClass), '.Init (aParent: pWindowsObject; anId: Integer; Title: pChar;'#13#10+
    '                      x,y,w,h: Integer; Len: LongInt; Data: Pointer);'#13#10+
    'Begin'#13#10+
    '  Inherited Init(aParent, anId, '''), VbxBaseName), ''', '''), VbxClass),
       ''', Title, x, y, w, h, Len, Data);'#13#10+
    'End;'#13#10+
          #13#10+
    'Constructor t'), VbxClass), '.InitResource(aParent: pWindowsObject; anId: Integer);'#13#10+
    'Begin'#13#10+
    '  Inherited InitResource(aParent, anId);'#13#10+
    'End;'#13#10#13#10));
  For i:= 0 To GetNumEvents-1 Do
    BufPtr:= StrEnd(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(StrCat(BufPtr,
      'Procedure t'), VbxClass), '.ev'), GetEventId(Tmp, i)), ' (Var Event: tVbxEvent);'#13#10+
      'Begin'#13#10+
      '  {$IfDef Debug} WriteLn(''[t'), VbxClass), '.ev'), GetEventId(Tmp, i)), ']''); {$EndIf}'#13#10+
      'End;'#13#10#13#10));
End;

Procedure tInfoWindow.RetrieveInfo (aVBXName: pChar);
Var
  ci: pVbxClass;
  p: pChar;
  Ctl: pDumpControl;
  ModName, PasName: Array[0..67] Of Char;
  aBuf: pChar;

  Function MakeLp (aPointer: Pointer; Index: Word): Pointer;
  Begin
    MakeLp:= Ptr(PtrRec(aPointer).Seg, Index)
  End;

Begin
  dVbx.VbxLoadVbx(aVBXName);
  StrJustName(ModName, aVbxName);
  p:= StrScan(ModName, '.'); If Assigned(p) Then p^:= #0;
  StrCat(StrCopy(PasName, ModName),'.Pas');
  SetFileName(PasName);
  aBuf:= MemAlloc(64000);
  p:= StrEnd(
  StrCat(StrCat(StrCopy(aBuf, 'Unit '), ModName),';'#13#10+
         '{this file was automatically generated by VbxInfo.'#13#10+
         ' VbxInfo is (c) 1994 Peter Sawatzki}'#13#10+
         'Interface'#13#10+
         'Uses'#13#10+
         '  oWindows,'#13#10+
         '  Vbx;'#13#10));

  ci:= dVBX.VbxGetFirstClass;
  While Assigned(ci) Do With ci^, ClassInfo^ Do Begin
    Ctl:= New(pDumpControl, Init(@Self, aVbxName, MakeLp(ClassInfo, npszClassName), p));
    With Ctl^ Do Begin
      Create;
      DumpProperties;
      DumpEvents;
      DumpObjectDefinition;
    End;
    Dispose(Ctl, Done);
    ci:= dVbx.VbxGetNextClass(ci)
  End;
  p:= StrEnd(StrCat(aBuf, #13#10'Implementation'#13#10));

  ci:= dVBX.VbxGetFirstClass;
  While Assigned(ci) Do With ci^, ClassInfo^ Do Begin
    Ctl:= New(pDumpControl, Init(@Self, aVbxName, MakeLp(ClassInfo, npszClassName), p));
    With Ctl^ Do Begin
      Create;
      DumpObjectImplementation;
    End;
    Dispose(Ctl, Done);
    ci:= dVbx.VbxGetNextClass(ci)
  End;
  StrCat(aBuf, 'End.');

  Editor^.Clear;
  Editor^.Insert(aBuf);
  FreeMem(aBuf, 64000)
End;

{-------------------- the Application part }
Const
  ProgName = 'VbxInfo';
Type
  tProgApp = Object(tApplication)
    Procedure InitMainWindow; Virtual;
  End;

Procedure tProgApp.InitMainWindow;
Begin
  MainWindow:= New(pInfoWindow, Init(Nil, ProgName))
End;

Var
  App: tProgApp;
Begin
  RegisterVBX(VBXvalidation);
  With App Do Begin
    Init(ProgName);
    Run;
    Done
  End
End.

